#ifndef __LIBWR_GPIO_H
#define __LIBWR_GPIO_H

#include <stdint.h>

#define GPIO_OUT 1
#define GPIO_IN 0

struct gpio_pin;

typedef int (*gpio_configure_func_t)(const struct gpio_pin *);
typedef void (*gpio_set_dir_func_t)(const struct gpio_pin *, int);
typedef void (*gpio_set_out_func_t)(const struct gpio_pin *, int);
typedef int (*gpio_read_pin_func_t)(const struct gpio_pin *);

struct gpio_device
{
    void* priv;
    gpio_set_dir_func_t set_dir;
    gpio_set_out_func_t set_out;
    gpio_read_pin_func_t read_pin;
    gpio_read_pin_func_t configure;
} gpio_device_t;

typedef struct gpio_pin
{
    const struct gpio_device *dev;
    int pin;
    void *priv;
} gpio_pin_t;


int shw_gpio_configure(const struct gpio_pin * pin)
{
    return pin->dev->configure( pin );
}

static inline void shw_gpio_set(struct gpio_pin *pin, int state)
{
    pin->dev->set_out( pin, state );
}

static inline void shw_gpio_set1(struct gpio_pin *pin)
{
    pin->dev->set_out( pin, 1 );
}

static inline void shw_gpio_set0(struct gpio_pin *pin)
{
    pin->dev->set_out( pin, 0 );
}

static inline int shw_gpio_get(struct gpio_pin *pin)
{
    return pin->dev->read_pin( pin );
}

static inline void shw_gpio_set_dir(struct gpio_pin* pin, int dir)
{
    pin->dev->set_dir( pin, dir );
}


#endif /* __LIBWR_GPIO_H */
