#include <stdio.h>
#include <errno.h>
#include <termios.h>
#include <unistd.h>
#include <stdlib.h>
#include <sys/select.h>

#include <wrs-hw.h>
#include <libwr/switch_hw.h>
#include <fpga_io.h>

#include "libwr/i2c.h"
#include "libwr/i2c_fpga_reg.h"

i2c_fpga_reg_t fpga_bus0_reg = {
	.base_address = FPGA_I2C_ADDRESS,
	.if_num = FPGA_I2C0_IFNUM,
	.prescaler = 500,
};

struct i2c_bus i2c_bus_fpga = {
	.name = "fpga_bus0",
	.type = I2C_BUS_TYPE_FPGA_REG,
	.type_specific = &fpga_bus0_reg};

#define GPIO_COR 0x0 // fixme. alessandro, I hate you!
#define GPIO_SOR 0x4
#define RT_PERIPH_RESET_PIN_N 3


int test_i2c()
{
	printf("Unreset periph...\n");
	_fpga_writel(WRS_FPGA_BASE_RTS_GPIO + GPIO_SOR, 1 << RT_PERIPH_RESET_PIN_N); // un-reset the peripherals

	uint8_t devs[256];
	i2c_bus_init(&i2c_bus_fpga);
	
	memset(devs, 0, sizeof(devs));
	i2c_bus_scan(&i2c_bus_fpga, devs);

	int i;
	printf("Found devs: ");
	for (i = 0; i < 128; i++)
		if (devs[i / 8] & (1<<(i&0x7)))
			printf("%02x ", i);
	printf("\n");

	return 0;
}

int main(int argc, char *argv[])
{
	wrs_msg_init( argc, argv, 0 );

	shw_fpga_mmap_init();
	test_i2c();
	return 0;
}
